@@ -1,12 +1,12 @@
 plspm <-
-function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
-                  boot.val=FALSE, br=NULL, plsr=FALSE, tol=0.00001, iter=100)
+function(x, inner, outer, modes=NULL, scheme="centroid", scaled=TRUE,
+            boot.val=FALSE, br=NULL, plsr=FALSE, tol=0.00001, iter=100, dataset=TRUE)
 {
     # =========================== ARGUMENTS ====================================
     # x: a numeric matrix or data.frame containing the manifest variables
-    # inner.mat: a square boolean matrix indicating the path relations
-    # sets: a list of vectors of indices indicating the manifest variables 
-    #       for each block
+    # inner: a square boolean matrix representing the inner model
+    # outer: a list representing the outer model (list of vectors)
+    #        from matrix x with manifest variables for each block
     # modes: a character vector indicating the measurement type for each 
     #        latent variable. "A" reflective, "B" formative
     # scheme: a character string indicating the inner weighting scheme 
@@ -18,6 +18,7 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
     # plsr: a logical value for calculating path coeffs by pls-regression
     # tol: tolerance threshold for calculating outer weights (0.00001)
     # iter: an integer indicating the maximum number of iterations (100)
+    # dataset: a logical value indicating whether the data should be retrieved
     # ===========================================================================
 
     # ==================== Checking function arguments ===================
@@ -27,35 +28,35 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
         rownames(x) <- 1:nrow(x)
     if (is.null(colnames(x))) 
         colnames(x) <- paste("MV", 1:ncol(x), sep="")
-    if (!is.matrix(inner.mat))
-        stop("Invalid argument 'inner.mat'. Must be a matrix.")
-    if (nrow(inner.mat)!=ncol(inner.mat))
-        stop("Invalid argument 'inner.mat'. Must be a square matrix.")
-    for (j in 1:ncol(inner.mat))
-        for (i in 1:nrow(inner.mat)) {
+    if (!is.matrix(inner))
+        stop("Invalid argument 'inner'. Must be a matrix.")
+    if (nrow(inner)!=ncol(inner))
+        stop("Invalid argument 'inner'. Must be a square matrix.")
+    for (j in 1:ncol(inner))
+        for (i in 1:nrow(inner)) {
             if (i<=j)
-                if (inner.mat[i,j]!=0) 
-                    stop("argument 'inner.mat' must be a lower triangular matrix")
-            if (length(intersect(inner.mat[i,j], c(1,0)))==0)
-                stop("elements in 'inner.mat' must be '1' or '0'")
+                if (inner[i,j]!=0) 
+                    stop("argument 'inner' must be a lower triangular matrix")
+            if (length(intersect(inner[i,j], c(1,0)))==0)
+                stop("elements in 'inner' must be '1' or '0'")
         }
-    if (is.null(dimnames(inner.mat)))
-        lvs.names <- paste("LV", 1:ncol(inner.mat), sep="")
-    if (!is.null(rownames(inner.mat)))
-        lvs.names <- rownames(inner.mat)
-    if (!is.null(colnames(inner.mat)))
-        lvs.names <- colnames(inner.mat)
-    if (!is.list(sets))
-        stop("Invalid argument 'sets'. Must be a list.")
-    if (length(sets) != nrow(inner.mat))
-        stop("Number of rows of 'inner.mat' does not coincide with length of 'sets'.")
+    if (is.null(dimnames(inner)))
+        lvs.names <- paste("LV", 1:ncol(inner), sep="")
+    if (!is.null(rownames(inner)))
+        lvs.names <- rownames(inner)
+    if (!is.null(colnames(inner)))
+        lvs.names <- colnames(inner)
+    if (!is.list(outer))
+        stop("Invalid argument 'outer'. Must be a list.")
+    if (length(outer) != nrow(inner))
+        stop("Number of rows of 'inner' does not coincide with length of 'outer'.")
     if (is.null(modes)) {
-        modes <- rep("A",length(sets))
+        modes <- rep("A",length(outer))
         warning("Argument 'modes' missing. Default reflective 'modes' is used.")
     }
-    if (length(sets) != length(modes)) {
-        warning("Invalid length of 'modes'. Default reflective 'modes' is used.")
-        modes <- rep("A", length(sets))
+    if (length(outer) != length(modes)) {
+        warning("Warning: Invalid length of 'modes'. Default reflective 'modes' is used.")
+        modes <- rep("A", length(outer))
     }
     for (i in 1:length(modes))
         if (modes[i]!="A" && modes[i]!="B") modes[i]<-"A"
@@ -64,22 +65,22 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
     SCHEMES <- c("centroid", "factor", "path")
     scheme <- pmatch(scheme, SCHEMES)
     if (is.na(scheme)) {
-        warning("Invalid argument 'scheme'. Default 'scheme=centroid' is used.")   
+        warning("Warning: Invalid argument 'scheme'. Default 'scheme=centroid' is used.")   
         scheme <- "centroid"
     }
     if (!is.logical(scaled)) {
-        warning("Invalid argument 'scaled'. Default 'scaled=TRUE' is used.")
+        warning("Warning: Invalid argument 'scaled'. Default 'scaled=TRUE' is used.")
         scaled <- TRUE
     }
     if (!is.logical(boot.val)) {
-        warning("Invalid argument 'boot.val'. No bootstrap validation is done.")
+        warning("Warning: Invalid argument 'boot.val'. No bootstrap validation is done.")
         boot.val <- FALSE
     }   
     if (boot.val) {
         if (!is.null(br)) {        
             if (mode(br)!="numeric" || length(br)!=1 || (br%%1)!=0 ||
                 br<100 || br>1000) {
-                warning("Invalid argument 'br'. Default 'br=100' is used.")   
+                warning("Warning: Invalid argument 'br'. Default 'br=100' is used.")   
                 br <- 100
             } 
         } else
@@ -87,24 +88,26 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
     }
     if (!is.logical(plsr)) plsr<-FALSE
     if (mode(tol)!="numeric" || length(tol)!=1 || tol<=0 || tol>0.001) {
-        warning("Invalid argument 'tol'. Default 'tol=0.00001' is used.")   
+        warning("Warning: Invalid argument 'tol'. Default 'tol=0.00001' is used.")   
         tol <- 0.00001
     } 
     if (mode(iter)!="numeric" || length(iter)!=1 || iter<100) {
-        warning("Invalid argument 'iter'. Default 'iter=100' is used.")   
+        warning("Warning: Invalid argument 'iter'. Default 'iter=100' is used.")   
         iter <- 100
     } 
+    if (!is.logical(dataset)) 
+        dataset <- TRUE
 
     # ========================== INPUTS SETTING ==========================
-    IDM <- inner.mat
+    IDM <- inner
     dimnames(IDM) <- list(lvs.names, lvs.names)
     lvs <- nrow(IDM)
     lvs.names <- rownames(IDM)
-    mvs <- sum(unlist(lapply(sets, length)))
-    blocks <- unlist(lapply(sets, length))
+    blocks <- unlist(lapply(outer, length))
+    mvs <- sum(blocks)
     names(blocks) <- lvs.names
-    blocklist <- sets
-    for (k in 1:length(sets))
+    blocklist <- outer
+    for (k in 1:length(blocks))
          blocklist[[k]] <- rep(k,blocks[k])
     blocklist <- unlist(blocklist)
     Mode <- modes
@@ -115,8 +118,8 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
     mvs.names <- rep(NA, mvs)
     for (k in 1:lvs)
     {        
-        DM[,which(blocklist==k)] <- as.matrix(x[,sets[[k]]])
-        mvs.names[which(blocklist==k)] <- colnames(x)[sets[[k]]]
+        DM[,which(blocklist==k)] <- as.matrix(x[,outer[[k]]])
+        mvs.names[which(blocklist==k)] <- colnames(x)[outer[[k]]]
     }
     dimnames(DM) <- list(rownames(x), mvs.names)
     # apply the selected scaling
@@ -132,7 +135,8 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
     out.ws <- .pls.weights(X, IDM, blocks, modes, scheme, tol, iter)
     if (is.null(out.ws)) {
         print(paste("Iterative process is non-convergent with 'iter'=", iter, " and 'tol'=", tol, sep=""))
-        stop("Algorithm stops") 
+        message("Algorithm stops") 
+        stop("")
     }
     out.weights <- out.ws[[1]]
     cor.XY <- cor(X, X%*%out.ws[[2]])
@@ -203,11 +207,12 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
     skem <- switch(scheme, "centroid"="centroid", "factor"="factor", "path"="path")
     model <- list(IDM=IDM, blocks=blocks, scheme=skem, modes=modes, scaled=scaled, 
                   boot.val=boot.val, plsr=plsr, obs=nrow(X), br=br, 
-                  tol=tol, iter=iter, n.iter=out.ws[[3]])
+                  tol=tol, iter=iter, n.iter=out.ws[[3]], outer=outer)
+    if (dataset) data=DM else data=NULL
     res <- list(outer.mod=outmod, inner.mod=innmod, latents=Z.lvs, scores=Y.lvs,
                out.weights=out.weights, loadings=loads, path.coefs=Path, r.sqr=R2,
                outer.cor=outcor, inner.sum=innsum, effects=Path.efs, unidim=unidim, gof=gof, 
-               data=DM, model=model)
+               data=data, model=model)
     # ========================= Bootstrap Validation =========================
     if (boot.val) 
     {
@@ -219,9 +224,9 @@ function(x, inner.mat, sets, modes=NULL, scheme="centroid", scaled=TRUE,
             res.boot <- .pls.boot(DM, IDM, blocks, modes, scheme, scaled, br, plsr, tol, iter)
         }
         res <- list(outer.mod=outmod, inner.mod=innmod, latents=Z.lvs, scores=Y.lvs,
-                 out.weights=out.weights, loadings=loads, path.coefs=Path, r.sqr=R2,
-                 outer.cor=outcor, inner.sum=innsum, effects=Path.efs, unidim=unidim, gof=gof, 
-                 boot=res.boot, data=DM, model=model)
+                   out.weights=out.weights, loadings=loads, path.coefs=Path, r.sqr=R2,
+                   outer.cor=outcor, inner.sum=innsum, effects=Path.efs, unidim=unidim, gof=gof, 
+                   boot=res.boot, data=data, model=model)
     } # end 'if' bootstrapping
     # --------------------------------------------------------------------
     class(res) <- "plspm"
